;( function( $, window, document, undefined ) {
    "use strict";

    var pluginName = "liveArticle";
    var nids       = [];
    var elements   = {
        'live' : null,
        'count': null,
        'clock': null
    };
    var defaults   = {
        'interval': 10000
    };

    function Plugin ( element, options ) {
        this.element   = $(element);
        this.settings  = $.extend( {}, defaults, options );
        this._defaults = defaults;
        this._name     = pluginName;
        this.nid       = this.element.attr("data-live-nid");
        this.template  = Handlebars.compile(
            $('<div>')
            .append(
                this.element.find('[data-template="live-entry"]:first')
                    .remove()
                    .removeAttr("data-template")
            ).html()
        );

        if (this.element.attr("data-live-node")) {
            this.init();
        }
    }

    function fetch () {
        for (var n=0; n < nids.length; n++) {
            $.get('/sites/lexpress/files/live/' + nids[n] + '.json', function (response) {
                elements.live.filter("[data-live-nid='" + response.nid + "']").trigger("LIVE_UPDATED", [response]);
                window.setTimeout(fetch, defaults.interval);
            });
        }
    }

    $.extend( Plugin.prototype, {
        init: function() {
            if (nids.indexOf(this.nid) === -1) {
                nids.push(this.nid);
            }

            this.element.on("LIVE_UPDATED", this.update.bind(this));
        },
        update: function(event, response) {
            var i, entry, is_updated, last_updated = "0000-00-00 00:00", total_new = 0;
            var process = function(entry) {
                var element  = $(this.template(entry));
                var isnew    = 0;

                var existing = this.element.find('[data-live-entry]')
                    .removeClass("updated")
                    .filter('[data-live-entry="' + entry.nid + '"]');

                element.parent().find('[data-last-updated]')
                    .html(entry.time);

                if (existing.size()) {
                    is_updated = entry.datetime > existing.attr('data-live-time');
                    if (is_updated) {
                        existing.replaceWith(element);
                    }
                } else {
                    this.element.prepend(element);
                    element.addClass("updated");
                    isnew = 1;
                }

                element.find('[data-live-link="entry"]').on('click', function(event) {
                    event.preventDefault();
                    goToByScroll.call(this);
                });

                return isnew;
            }.bind(this);

            for (i = response.entries.length - 1; i >= 0; i--) {
                total_new += process(response.entries[i]);
                if (last_updated < response.entries[i].datetime) {
                    last_updated = response.entries[i].datetime;
                }
            }

            elements.count.filter("[data-live-count-nid='" + response.nid + "']")
                .each(function(){
                    $(this).html(total_new);
                });

            elements.clock.filter("[data-live-clock-nid='" + response.nid + "']")
                .each(function(){
                    var me = $(this);
                    me.html(last_updated).attr('datetime', last_updated);
                    if (last_updated === '0000-00-00 00:00') {
                        me.hide();
                    } else {
                        me.fadeIn();
                    }
                });
        }
    } );

    $.fn[ pluginName ] = function( options ) {
        return this.each( function() {
            if ( !$.data( this, "plugin_" + pluginName ) ) {
                $.data( this, "plugin_" +
                    pluginName, new Plugin( this, options ) );
            }
        } );
    };

    $(document).ready(function() {
        elements.live = $('[data-live-nid]');
        elements.live[pluginName]();

        elements.count = $('[data-live-count-nid]');
        elements.clock = $('[data-live-clock-nid]');
        fetch();
    });

} )( $, window, document );
